# BurnMan - a lower mantle toolkit
# Copyright (C) 2012, 2013, Heister, T., Unterborn, C., Rose, I. and Cottaar, S.
# Released under GPL v2 or later.

"""
Mineral database
"""

import Murakami_etal_2012
import Matas_etal_2007
import SLB_2011
import SLB_2011_ZSB_2013
import SLB_2005
import Murakami_2013
import other
